@@ -1,8 +1,6 @@
 /*
- * $RCSfile: BlockSelectWidget.js,v $
- *
  * Gallery - a web based photo album viewer and editor
- * Copyright (C) 2000-2006 Bharat Mediratta
+ * Copyright (C) 2000-2007 Bharat Mediratta
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +19,8 @@
 /*
  * Author: Ariel Garza (primary development)
  * Author: Bharat Mediratta (minor adjustments for namespacing, bugfixes)
+ * NOTE: If the blocks data format changes, bump the "version" number in URLs
+ *       referencing this file to ensure old/cached code is not used.
  */
 var bsw_WIDGET_BLOCKS = new Array();
 var bsw_FORM_TARGETS = new Array();
@@ -93,17 +93,27 @@ function bsw_BlockPreference(key, blockId, values) {
     };
 
     this.toString = function() {
-	var result = "[" + this.blockId;
+	var result = '["' + this.blockId + '",{';
+	var count = 0;
 	for (var paramName in this.parameters) {
 	    var paramValue = this.parameters[paramName];
 	    var defaultValue =
 		bsw_WIDGET_BLOCKS[key][blockId]['parameters'][paramName]['defaultValue'];
+	    var paramType = bsw_WIDGET_BLOCKS[key][blockId]['parameters'][paramName]['type'];
 	    if (paramValue != defaultValue) {
-		result += " " + paramName + "=" + paramValue;
+		if (count++) {
+		    result += ',';
+		}
+
+		result += '"' + paramName + '":';
+		if (paramType == 'text' || paramType == 'choice') {
+		    result += '"' + paramValue.replace(/\"/g,'\\"') + '"';
+		} else {
+		    result += paramValue;
+		}
 	    }
 	}
-
-	result += "]";
+	result += '}]';
 
 	return result;
     };
@@ -155,7 +165,15 @@ function bsw_addUserBlockPreference(key, blockId, values) {
 function bsw_initAdminForm(key, parameterText, valueText) {
     var availableEl = document.getElementById("blocksAvailableList_" + key);
     var usedEl = document.getElementById("blocksUsedList_" + key);
-    var usedBlockString = document.getElementById("albumBlockValue_" + key).value;
+
+    /*
+     * Undo the " -> &quot; coercion that we had to do in order to embed the
+     * serialized value into a string value in HTML.
+     */
+    var valueEl = document.getElementById("albumBlockValue_" + key);
+    valueEl.value = valueEl.value.replace('&quot;', '"');
+    var usedBlockString = valueEl.value;
+
     bsw_createBlockPrefsFromString(key, usedBlockString);
     bsw_resizeBlocksUsedList(key);
     bsw_selectNone(key);
@@ -173,7 +191,7 @@ function bsw_reInitAdminForm(key) {
     if (bsw_UPDATE_LEVEL == 0) {
 	var usedBlockString = document.getElementById("albumBlockValue_" + key).value;
 	bsw_USER_BLOCK_POSITIONS[key] = new Array();
-	bsw_createBlockPrefsFromString(key, usedBlockString);
+	bsw_createBlockPrefsFromString(key, unescape(usedBlockString));
 	bsw_selectNone(key);
     }
 }
@@ -236,7 +254,7 @@ function bsw_selectToChange(key) {
     if (usedEl.selectedIndex > 0) {
 	bsw_enableButton(key, "MoveUpButton");
     } else {
-        bsw_disableButton(key, "MoveUpButton");
+	bsw_disableButton(key, "MoveUpButton");
     }
 
     if (usedEl.selectedIndex < usedEl.options.length - 1) {
@@ -407,7 +425,7 @@ function bsw_showBlockOptions(key) {
     /* Now add the parameter value rows */
     callbacks = new Array();
     for (var paramName in block.parameters) {
-        var paramRowEl = document.createElement("tr");
+	var paramRowEl = document.createElement("tr");
 
 	var paramNameEl = document.createElement("td");
 	paramNameEl.appendChild(document.createTextNode(block.parameters[paramName]['description']));
@@ -488,6 +506,13 @@ function bsw_getValueElement(paramName, blockPref, block) {
 	}
 	returnElement.selectedIndex = -1;
 
+	returnElement.onchange = function() { bsw_updatePrefValue(block.key, elementType, this); };
+    } else if (elementType == 'text') {
+	returnElement = document.createElement("input");
+	returnElement.type = 'text';
+	callback = new Function('document.getElementById("' + id + '").value = "' +
+				prefValue.replace(/\"/g, '\\"') + '";');
+	returnElement.value = prefValue;
 	returnElement.onchange = function() { bsw_updatePrefValue(block.key, elementType, this); };
     }
 
@@ -515,8 +540,11 @@ function bsw_updatePrefValue(key, varType, element) {
     var paramValue;
     if (varType == 'boolean') {
 	paramValue = element.checked;
-    } else {
+    } else if (varType == 'choice') {
 	paramValue = element.options[element.selectedIndex].value;
+    } else {
+	/* text */
+	paramValue = element.value;
     }
 
     blockPref.setParameterValue(paramName, paramValue);
@@ -540,9 +568,6 @@ function bsw_updateVarOverrides(key, blockPref) {
     }
 }
 
-/* Create a regexp pattern that will pull out all of the block definitions */
-var BLOCK_REGEXP = /\[[^\]]+\]/g;
-
 /**
  * Given a string value as stored in the gallery database, create block
  * preference objects with the supplied values.
@@ -557,24 +582,10 @@ function bsw_createBlockPrefsFromString(key, stringValue) {
 
     /* Now clear out the block array */
     bsw_USER_BLOCKS[key] = new Array();
+    eval('var blockData = ' + stringValue);
 
-    var result = stringValue.match(BLOCK_REGEXP);
-    if (result == null) {
-	return;
-    }
-
-    /* At this point we have a bunch of matches with the enclosing []'s */
-    for (var i = 0; i < result.length; i++) {
-	var bits = result[i].substring(1, result[i].length - 1).split(" ");
-
-        var blockId = bits[0];
-	var prefValues = new Array();
-	for (var bitIndex = 1; bitIndex < bits.length; bitIndex++) {
-	    var paramBits = bits[bitIndex].split("=");
-	    prefValues[paramBits[0]] = paramBits[1];
-	}
-
-	bsw_addUserBlockPreference(key, blockId, prefValues);
+    for (var i = 0; i < blockData.length; i++) {
+	bsw_addUserBlockPreference(key, blockData[i][0], blockData[i][1]);
     }
 }
 
@@ -584,8 +595,9 @@ function bsw_createBlockPrefsFromString(key, stringValue) {
  */
 function bsw_updateAlbumBlockValue(key) {
     var albumValueEl = document.getElementById("albumBlockValue_" + key);
-    var value = "";
+    var newValue = '[';
 
+    var count = 0;
     for (var i = 0; i < bsw_USER_BLOCK_POSITIONS[key].length; i++) {
 	var blockPrefId = bsw_USER_BLOCK_POSITIONS[key][i];
 	var blockPref = bsw_USER_BLOCKS[key][blockPrefId];
@@ -595,16 +607,19 @@ function bsw_updateAlbumBlockValue(key) {
 	 * when the old block string contained blocks that are no longer in the master list.
 	 */
 	if (bsw_WIDGET_BLOCKS[key][blockPref.blockId]) {
-	    value += blockPref.toString() + " ";
+	    if (count++) {
+		newValue += ',';
+	    }
+	    newValue += blockPref.toString();
 	}
     }
+    newValue += ']';
 
     /*
      * Don't change it unless we really need to, to avoid unnecessarily
      * propagating onchange events which can lead to recursive calls back
      * to bsw_updateAlbumBlockValue()
      */
-    var newValue = value.substring(0, value.length - 1);
     if (albumValueEl.value != newValue) {
 	albumValueEl.value = newValue;
 	if (albumValueEl.onchange) {
