@@ -1,9 +1,7 @@
 <?php
 /*
- * $RCSfile: generate-sql.php,v $
- *
  * Gallery - a web based photo album viewer and editor
- * Copyright (C) 2000-2006 Bharat Mediratta
+ * Copyright (C) 2000-2007 Bharat Mediratta
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +26,7 @@ if (!empty($_SERVER['SERVER_NAME'])) {
 require_once(dirname(__FILE__) . '/XmlParser.inc');
 
 $output = '';
-foreach (array('mysql', 'postgres', 'oracle', 'db2') as $db) {
+foreach (array('mysql', 'postgres', 'oracle', 'db2', 'mssql') as $db) {
     $output .= '## ' . $db . "\n";
     $xmlFiles = glob('tmp/dbxml/*.xml');
     if (empty($xmlFiles)) {
@@ -230,7 +228,7 @@ class MySqlGenerator extends BaseGenerator {
 		}
 		$output .= "\n";
 	    }
-	    $output .= ") TYPE=DB_TABLE_TYPE\n";
+	    $output .= ") DB_TABLE_TYPE\n";
 	    /* Character set, enclosed in comments that are ignored by MySQL < 4.1.0 */
 	    $output .= "/*!40100 DEFAULT CHARACTER SET utf8 */;\n\n";
 
@@ -960,6 +958,23 @@ class Db2Generator extends BaseGenerator {
 		'TIMESTAMP-' => 'datestamp'));
     }
 
+    function columnDefinition($child, $includeNotNull=true, $includeDefault=true) {
+	$output = parent::columnDefinition($child, $includeNotNull, false);
+
+	/* DB2 -> Make sure DEFAULT expression doesn't have quotes for numeric */
+	if ($includeDefault) {
+	    $defaultValue = $this->getDefaultElement($child);
+	    if (isset($defaultValue)) {
+		if ($child[1]['content'] != 'INTEGER' && $child[1]['content'] != 'BOOLEAN') {
+		    $defaultValue = "'$defaultValue'";
+		}
+		$output .= " DEFAULT $defaultValue";
+	    }
+	}
+
+	return $output;
+    }
+
     function createSql($node, $index, $lastPeerIndex, $parent) {
 	$output = '';
 
@@ -1184,4 +1199,278 @@ class Db2Generator extends BaseGenerator {
     }
 }
 
+class MSSqlGenerator extends BaseGenerator {
+    function MSSqlGenerator() {
+	$this->setColumnDefinitionMap(
+	    array(
+		'INTEGER-' => 'INT',
+		'INTEGER-MEDIUM' => 'INT',
+		'INTEGER-LARGE' => 'INT',
+		'BIT-LARGE' => 'INT',
+		'BIT-MEDIUM' => 'INT',
+		'STRING-SMALL' => 'NVARCHAR(32)',
+		'STRING-MEDIUM' => 'NVARCHAR(128)',
+		'STRING-LARGE' => 'NVARCHAR(255)',
+		'TEXT-SMALL' => 'NVARCHAR(MAX)',
+		'TEXT-' => 'NVARCHAR(MAX)',
+		'TEXT-MEDIUM' => 'NVARCHAR(MAX)',
+		'TEXT-LARGE' => 'NVARCHAR(MAX)',
+		'BOOLEAN-' => 'BIT',
+		'BOOLEAN-MEDIUM' => 'BIT',
+		'TIMESTAMP-' => 'datetime'));
+    }
+
+    function columnDefinition($child, $includeNotNull=true, $includeDefault=true) {
+	$output = parent::columnDefinition($child, $includeNotNull, $includeDefault);
+
+	if (!$this->getNotNullElement($child)) {
+	      $output .= ' NULL';
+	}
+
+	return $output;
+    }
+
+    function createSql($node, $index, $lastPeerIndex, $parent) {
+	$output = '';
+
+	$child = $node['child'] = isset($node['child']) ? $node['child'] : array();
+	switch($node['name']) {
+	case 'CHANGE':
+	    /* table-name, schema-from, schema-to, (add, alter, remove)+ */
+	    for ($i = 3; $i < count($child); $i++) {
+		$output .= $this->createSql($child[$i], $i, count($child) - 1, $node);
+	    }
+	    $output .= $this->generateSchemaUpdate($child);
+	    break;
+
+	case 'REMOVE':
+	    if (!isset($parent['name'])) {
+		$output .= 'DROP TABLE DB_TABLE_PREFIX' . $node['child'][0]['content'] . ";\n\n";
+		if ($node['child'][0]['content'] != 'Schema') {
+		    $output .= "DELETE FROM DB_TABLE_PREFIXSchema WHERE DB_COLUMN_PREFIXname='" .
+			$node['child'][0]['content'] . "';\n\n";
+		}
+	    } else if ($parent['name'] == 'CHANGE') {
+		/* (column-name, key, index)+ */
+		for ($i = 0; $i < count($child); $i++) {
+		    $c = $child[$i];
+		    switch($c['name']) {
+		    case 'COLUMN-NAME':
+			/* column-name */
+			$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'];
+			$output .= ' DROP COLUMN DB_COLUMN_PREFIX' . $c['content'];
+			$output .= ";\n\n";
+			break;
+
+		    case 'KEY':
+			if (empty($c['attrs']['PRIMARY'])) {
+			    $crc = $this->getIndexCrc($c['child']);
+			    $output .= 'DROP INDEX DB_TABLE_PREFIX' .
+				$parent['child'][0]['content'] . '_' . $crc . ";\n\n";
+			} else {
+			    $output .= 'ALTER TABLE DB_TABLE_PREFIX' .
+				$parent['child'][0]['content'] . ' DROP CONSTRAINT DB_TABLE_PREFIX'
+				. $parent['child'][0]['content'] . "_pkey;\n\n";
+			}
+			break;
+
+		    case 'INDEX':
+			/* column-name */
+			$output .= 'DROP INDEX ';
+			$nameKey = strtoupper('name_' . $this->getDbType());
+			if (isset($c['attrs'][$nameKey])) {
+			    $output .= $c['attrs'][$nameKey];
+			} else {
+			    $output .= 'DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+				'.DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+				'_' . $this->getIndexCrc($c['child']);
+			}
+			$output .= ";\n\n";
+			break;
+
+		    default:
+			$output .= "5. UNIMPLEMENTED: REMOVE $c[name]\n";
+		    }
+		}
+	    }
+	    break;
+
+	case 'ADD':
+	    /* (column, key, index)+ */
+	    foreach ($child as $c) {
+		switch($c['name']) {
+		case 'COLUMN':
+		    /* column-name */
+		    $output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'];
+		    $output .= ' ADD DB_COLUMN_PREFIX' . $c['child'][0]['content'];
+		    $output .= ' ' . $this->columnDefinition($c['child'], false, false);
+		    $output .= ";\n\n";
+
+		    if ($this->getNotNullElement($c['child'])) {
+			$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+			    ' ALTER COLUMN DB_COLUMN_PREFIX' . $c['child'][0]['content'];// .
+
+			    $key = $c['child'][1]['content'] . '-' .
+			      (!empty($c['child'][2]['content']) ? $c['child'][2]['content'] : '');
+			    if (isset($this->_columnDefinitionMap[$key])) {
+				$output .= ' ' . $this->_columnDefinitionMap[$key];
+			    } else {
+				$output .= ' ' . "2. UNIMPLEMLENTED: $key";
+			    }
+
+			    $output .= " NOT NULL;\n\n";
+		    }
+
+		    $defaultValue = $this->getDefaultElement($c['child']);
+		    if (isset($defaultValue)) {
+			$crc = $this->getIndexCrc(array($c['child'][0]));
+			$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'];
+			$output .= ' ADD CONSTRAINT ';
+			$output .= 'DB_COLUMN_PREFIX' . $c['child'][0]['content'] . '_' . $crc;
+			$output .= " DEFAULT '$defaultValue' FOR ";
+			$output .= 'DB_COLUMN_PREFIX' . $c['child'][0]['content'] . ";\n\n";
+		    }
+		    break;
+
+		case 'KEY':
+		    /* column-name+ */
+		    $output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+			' ADD ';
+		    if (!empty($c['attrs']['PRIMARY'])) {
+			$output .= 'PRIMARY KEY(';
+		    } else {
+			$output .= 'UNIQUE KEY(';
+		    }
+		    for ($i = 0; $i < count($c['child']); $i++) {
+			$output .= 'DB_COLUMN_PREFIX' . $c['child'][$i]['content'];
+			if ($i < count($c['child']) - 1) {
+			    $output .= ', ';
+			}
+		    }
+		    $output .= ')';
+		    $output .= ";\n\n";
+		    break;
+
+		case 'INDEX':
+		    /* column-name */
+		    $output .= 'CREATE INDEX ';
+		    $nameKey = strtoupper('name_' . $this->getDbType());
+		    $columns = $c['child'];
+		    if (isset($c['attrs'][$nameKey])) {
+			$output .= $c['attrs'][$nameKey];
+		    } else {
+			$output .= 'DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+			    '_' . $this->getIndexCrc($columns);
+		    }
+		    $output .= ' ON ' . 'DB_TABLE_PREFIX' . $parent['child'][0]['content'] . '(';
+		    for ($i = 0; $i < count($columns); $i++) {
+			$output .= 'DB_COLUMN_PREFIX' . $columns[$i]['content'];
+			if ($i < count($columns) - 1) {
+			    $output .= ', ';
+			}
+		    }
+		    $output .= ')';
+		    $output .= ";\n\n";
+		    break;
+
+		default:
+		    $output .= "6. UNIMPLEMLENTED: ADD $c[name]\n";
+		}
+	    }
+	    break;
+
+	case 'TABLE':
+	    /* table-name, schema, column+, (key | index)* */
+	    $output .= 'CREATE TABLE DB_TABLE_PREFIX' . $child[0]['content'] . "(\n";
+	    for ($i = 2; $i < count($child); $i++) {
+		if ($child[$i]['name'] != 'COLUMN') {
+		    $output .= "\n";
+		    break;
+		}
+		if ($i > 2) {
+		    $output .= ",\n";
+		}
+		$output .= $this->createSql($child[$i], $i, count($child) - 1, $node);
+		$firstNonColumn = $i + 1;
+	    }
+	    $output .= ");\n\n";
+
+	    for ($i = $firstNonColumn; $i < count($child); $i++) {
+		if ($child[$i]['name'] == 'INDEX') {
+		    $crc = $this->getIndexCrc($child[$i]['child']);
+		    $output .= 'CREATE INDEX DB_TABLE_PREFIX' . $child[0]['content'] . '_' . $crc .
+			' ON DB_TABLE_PREFIX' . $child[0]['content'] . "(";
+		    for ($j = 0; $j < count($child[$i]['child']); $j++) {
+			$output .= 'DB_COLUMN_PREFIX' . $child[$i]['child'][$j]['content'];
+			if ($j < count($child[$i]['child']) - 1) {
+			    $output .= ", ";
+			}
+		    }
+		    $output .= ");\n\n";
+		} else /* key */ {
+		    if (!empty($child[$i]['attrs']['PRIMARY'])) {
+			$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $child[0]['content'] .
+			    ' ADD PRIMARY KEY (';
+			$columns = $child[$i]['child'];
+			for ($j = 0; $j < count($columns); $j++) {
+			    $output .= 'DB_COLUMN_PREFIX' . $columns[$j]['content'];
+			    if ($j < count($columns) - 1) {
+				$output .= ', ';
+			    }
+			}
+			$output .= ");\n\n";
+		    } else {
+			$crc = $this->getIndexCrc($child[$i]['child']);
+			$output .= 'CREATE UNIQUE INDEX DB_TABLE_PREFIX' . $child[0]['content'] .
+			    '_' . $crc . ' ON DB_TABLE_PREFIX' . $child[0]['content'] . "(";
+			for ($j = 0; $j < count($child[$i]['child']); $j++) {
+			    $output .= 'DB_COLUMN_PREFIX' . $child[$i]['child'][$j]['content'];
+			    if ($j < count($child[$i]['child']) - 1) {
+				$output .= ", ";
+			    }
+			}
+			$output .= ");\n\n";
+		    }
+		}
+	    }
+
+	    /* Schema info */
+	    $output .= $this->createSql($child[1], 0, 0, $node);
+	    break;
+
+	case 'ALTER':
+	    /* column+ */
+	    for ($i = 0; $i < count($child); $i++) {
+		$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+		    ' ADD DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] . 'Temp';
+		$output .=
+		    ' ' . $this->columnDefinition($child[$i]['child'], false) . ";\n\n";
+		$output .= 'UPDATE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+		    ' SET DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] . 'Temp' .
+		    ' = CAST(DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] . ' AS ' .
+		    $this->columnDefinition($child[$i]['child'], false) . ");\n\n";
+		$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+		    ' DROP DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] . ";\n\n";
+		$output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+		    ' RENAME DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] . 'Temp' .
+		    ' to DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] . ";\n\n";
+		if ($this->getNotNullElement($child[$i]['child'])) {
+		    $output .= 'ALTER TABLE DB_TABLE_PREFIX' . $parent['child'][0]['content'] .
+			' ALTER DB_COLUMN_PREFIX' . $child[$i]['child'][0]['content'] .
+			" SET NOT NULL;\n\n";
+		}
+	    }
+	    break;
+
+	default:
+	    $output .= parent::createSql($node, $index, $lastPeerIndex, $parent);
+	}
+
+	return $output;
+    }
+
+    function getDbType() {
+	return 'mssql';
+    }
+}
 ?>
