@@ -3,36 +3,54 @@
 # This script traverses the Gallery tree and creates a manifest file that
 # contains a list of checksums for files in the distribution.  The installer
 # verifies the integrity of the files before proceeding.
-# The Debug module also uses this file to figure out whether or not it's
-# safe to show you the contents of the file in stack traces.
 #
 use strict;
 use File::Basename;
 use Cwd;
 use String::CRC32;
-
+my $quiet = 0;
+my $path;
+foreach my $i (0 .. $#ARGV) {
+    if ('-q' eq $ARGV[$i]) {
+	$quiet = 1;
+    } elsif ('-p' eq $ARGV[$i] && $#ARGV > $i) {
+	$path = $ARGV[$i+1];
+	# Relative path which serves as filter
+	die "$path does not exist" unless (-e $path);
+	die "$path must be a directory" unless (-d $path);
+	die "$path must be a relative path to a plugin (e.g. modules/core)"
+	  unless ($path =~ m{^(modules|themes)(/\w+)?/?$});
+    }
+}
+sub quietprint {
+    if (!$quiet) {
+	my $string = shift;
+	print STDERR $string;
+    }
+}
 $| = 1;
 
-chdir(dirname($0) . '/../../..');
+# Current working directory must be gallery2 folder, else we assume that we are in lib/tools/bin/.
+chdir(dirname($0)  . '/../../..') unless (-e 'modules' or -e 'themes');
 my $basedir = cwd();
 
-# Get a list of every file committed to CVS.
+# Get a list of every file committed to Subversion.
 #
 my @entries = ();
-print STDERR "Finding all files...";
-&parseCvs($basedir, \@entries);
-print STDERR "\n";
+quietprint("Finding all files...\n");
+my $filterPath = defined($path) ? $path : "";
+&listSvn(\@entries, $filterPath);
+quietprint("\n");
 
 # Strip base dir, sort
-print STDERR "Sorting...";
-map(s{$basedir/}{}, @entries);
+quietprint("Sorting...");
 @entries = sort @entries;
-print STDERR "\n";
+quietprint("\n");
 
 # Split into sections
 #
 my %sections;
-print STDERR "Separating into sections...";
+quietprint("Separating into sections...");
 foreach my $file (@entries) {
   if ($file =~ m{^((modules|layouts|themes)/.*?)/}) {
     push(@{$sections{"$1/MANIFEST"}}, $file);
@@ -40,15 +58,25 @@ foreach my $file (@entries) {
     push(@{$sections{'MANIFEST'}}, $file);
   }
 }
-print STDERR "\n";
+quietprint("\n");
 
 # Now generate the checksum files
 #
-print STDERR "Generating checksums...";
+quietprint("Generating checksums...");
 my $changed = 0;
 my $total = 0;
 foreach my $manifest (keys %sections) {
+  my @old_lines = ();
+  my $oldContent = '';
+  my $oldRevision = '';
+  if (open(FD, "<$manifest")) {
+    @old_lines = <FD>;
+    close(FD);
+    $oldRevision = $1 if ($old_lines[0] =~ /Revision(: \d+\s*)\$/);
+    $oldContent = join('', @old_lines);
+  }
   open(my $out, ">$manifest.new") or die;
+  print $out '# $Revision' . "$oldRevision\$\n";
   print $out "# File crc32 crc32(crlf) size size(crlf)  or  R File\n";
   my @entries = @{$sections{$manifest}};
   my %deleted;
@@ -67,14 +95,14 @@ foreach my $manifest (keys %sections) {
       close($fd);
 
       my ($data_crlf, $size, $size_crlf);
+      $data_crlf = $data;
       if ($isBinary) {
-	$data_crlf = $data;
 	$size = $size_crlf = (stat($file))[7];
       } else {
-	if ($data =~ /\r/) {
-	  ($data_crlf = $data) =~ s/\r//g;
+	if ($data =~ /\r\n/) {
+	  $data =~ s/\r\n/\n/g;
 	} else {
-	  ($data_crlf = $data) =~ s/\n/\r\n/g;
+	  $data_crlf =~ s/\n/\r\n/g;
 	}
 	$size = length($data);
 	$size_crlf = length($data_crlf);
@@ -85,13 +113,7 @@ foreach my $manifest (keys %sections) {
       print $out "$file\t$cksum\t$cksum_crlf\t$size\t$size_crlf\n";
     }
   }
-  my $oldContent = '';
-  if (open(FD, "<$manifest")) {
-    my @old_lines = <FD>;
-
-    $oldContent = join('', @old_lines);
-    close(FD);
-
+  if (@old_lines) {
     foreach (@old_lines) {
       next if /^\#/;
       if (/^R\t(.*)$/) {
@@ -116,11 +138,11 @@ foreach my $manifest (keys %sections) {
   $changed += replaceIfNecessary($oldContent, $manifest, "$manifest.new");
   $total++;
 
-  print STDERR ".";
+  quietprint(".");
 }
-print STDERR "\n";
-printf(STDERR "Completed in %d seconds\n", time - $^T);
-printf(STDERR "Manifests changed: $changed (total: $total)\n");
+quietprint("\n");
+quietprint(sprintf("Completed in %d seconds\n", time - $^T));
+quietprint(sprintf("Manifests changed: $changed (total: $total)\n"));
 
 sub replaceIfNecessary {
   my ($oldContent, $oldFile, $newFile) = @_;
@@ -138,42 +160,26 @@ sub replaceIfNecessary {
   }
 }
 
-sub parseCvs {
-  my $activeDir = shift;
+sub listSvn {
   my $entries = shift;
-
-  my $cvsDir = "$activeDir/CVS";
-  die "missing $cvsDir" unless (-d $cvsDir);
-
+  my $filterPath = shift;
+  my %binaryList = ();
   local *FD;
-  open(FD, "<$cvsDir/Entries") or die;
+  open(FD, "svn propget --non-interactive -R svn:mime-type $filterPath |") or die;
   while (<FD>) {
-    next if /^D$/;
-
-    if (m|^D/([^/]*)/|) {
-      my $target = "$activeDir/$1";
-      if (-d $target) {
-	&parseCvs($target, $entries);
-      } if (-f $target) {
-	die "$target not a dir" if (! -d $target);
-      } else {
-	# CVS doesn't always get rid of D/ entries when the dir is
-	# pruned.  Ignore.
-      }
-    } elsif (m|/([^/]*)/([^/]*)/|) {
-      next if m{/MANIFEST$};
-      my $target;
-      if ($2 =~ '^-') {
-	# Deleted file
-	$target = "deleted:$activeDir/$1";
-      } else {
-	$target = "$activeDir/$1";
-	die "$target not a file" if (! -f $target);
-      }
-      push(@$entries, sprintf("%s@@%d", $target, m/-kb/));
-    } else {
-      die "Can't parse: $_";
-    }
+    split / - /;
+    $binaryList{$_[0]} = 1;
+  }
+  close FD;
+  open(FD, "svn status --non-interactive -v -q $filterPath |") or die;
+  while (<FD>) {
+    die "\n$_" unless /^(.).....\s*\d+\s+\d+\s+\S+\s+(.*)$/;
+    die "\n$2" unless (-e $2);
+    next unless (-f $2);
+    die "Check $1 status for $2" if ($1 ne ' ' and $1 ne 'D' and $1 ne 'M');
+    quietprint("Warning: $2 is locally modified\n") if ($1 eq 'M');
+    push(@$entries,
+      sprintf("%s%s@@%d", ($1 eq 'D' ? 'deleted:' : ''), $2, exists($binaryList{$2})));
   }
   close FD;
 }
