@@ -1,9 +1,7 @@
 <?php
 /*
- * $RCSfile
- *
  * Gallery - a web based photo album viewer and editor
- * Copyright (C) 2000-2006 Bharat Mediratta
+ * Copyright (C) 2000-2007 Bharat Mediratta
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +21,10 @@
 /*
  * Authors: Jens Tkotz, Bharat Mediratta
  */
+define('G2_SUPPORT_URL_FRAGMENT', '../../support/');
 require_once(dirname(__FILE__) . '/../../support/security.inc');
 ini_set('magic_quotes_runtime', false);
+set_time_limit(0);
 
 $type = 'summary';
 if (!empty($_REQUEST['type']) && $_REQUEST['type'] == 'detail') {
@@ -32,11 +32,12 @@ if (!empty($_REQUEST['type']) && $_REQUEST['type'] == 'detail') {
 } else if (php_sapi_name() == 'cli' && $argv[1] == 'detail') {
     $type = 'detail';
 }
+$precision = isset($_GET['precision']) ? (int)$_GET['precision'] : ($type == 'detail' ? 2 : 1);
+$pow = pow(10, $precision);
 
 $poFiles = findPoFiles('../../..');
-list ($reportData, $mostRecentPoDate) = parsePoFiles($poFiles);
+list ($reportData, $mostRecentPoDate, $totalTranslated) = parsePoFiles($poFiles);
 
-$precision = isset($_GET['precision']) ? (int)$_GET['precision'] : 1;
 require(dirname(__FILE__) . '/localization/main_' . $type . '.inc');
 exit;
 
@@ -69,8 +70,9 @@ function parsePoFiles($poFiles) {
      * Parse each .po file for relevant statistics and gather it together into a
      * single data structure.
      */
+    global $pow;
     $poData = $seenPlugins = $maxMessageCount = array();
-    $mostRecentPoDate = 0;
+    $mostRecentPoDate = $totalTranslated = 0;
     foreach ($poFiles as $poFile) {
 	if (!preg_match("|((?:\w+/)+)po/(\w{2}(?:_\w{2})?)\.po|", $poFile, $matches)) {
 	    continue;
@@ -99,12 +101,14 @@ function parsePoFiles($poFiles) {
 	 *
 	 * Untranslated:
 	 *   msgid "foo"
+	 *   msgid_plural "foos"
 	 *   msgstr[0] ""
 	 *   msgstr[1] ""
 	 *   msgstr[2] ""
 	 *
 	 * Translated:
 	 *   msgid "foo"
+	 *   msgid_plural "foos"
 	 *   msgstr[0] "bar1"
 	 *   msgstr[1] "bar2"
 	 *   msgstr[2] "bar3"
@@ -195,7 +199,7 @@ function parsePoFiles($poFiles) {
 	     */
 	    if (!empty($msgId)) {
 		if (preg_match('/^msgstr/', $line)) {
-		    if (preg_match('/^msgstr\s*""\s*$/', $line)) {
+		    if (preg_match('/^msgstr[\d\[\]]*\s*""\s*$/', $line)) {
 			$lastLineWasEmptyMsgStr = 1;
 		    } else {
 			if ($nextIsFuzzy) {
@@ -215,9 +219,10 @@ function parsePoFiles($poFiles) {
 
 	$total = $translated + $untranslated;
 	if (empty($total)) {
-	    $percentDone = 0;
+	    $percentDone = $exactPercentDone = 0;
 	} else {
-	    $percentDone = ($translated - $fuzzy) * 100 / $total;
+	    $percentDone = floor(($translated - $fuzzy) * 100 * $pow / $total) / $pow;
+	    $exactPercentDone = ($translated - $fuzzy) * 100 / $total;
 	}
 	$poData[$locale]['plugins'][$plugin] = array('translated' => $translated,
 						     'untranslated' => $untranslated,
@@ -225,7 +230,9 @@ function parsePoFiles($poFiles) {
 						     'fuzzy' => $fuzzy,
 						     'obsolete' => $obsolete,
 						     'percentDone' => $percentDone,
+						     'exactPercentDone' => $exactPercentDone,
 						     'name' => $plugin);
+	$totalTranslated += $translated - $fuzzy;
 
 	foreach (array('translated', 'untranslated', 'fuzzy', 'obsolete') as $key) {
 	    if (!isset($summary[$locale][$key])) {
@@ -252,6 +259,7 @@ function parsePoFiles($poFiles) {
 	    if (!isset($poData[$locale]['plugins'][$plugin])) {
 		$poData[$locale]['plugins'][$plugin]['missing'] = 1;
 		$poData[$locale]['plugins'][$plugin]['percentDone'] = 0;
+		$poData[$locale]['plugins'][$plugin]['exactPercentDone'] = 0;
 		$poData[$locale]['plugins'][$plugin]['name'] = $plugin;
 	    } else {
 		$pluginTotal +=
@@ -263,13 +271,16 @@ function parsePoFiles($poFiles) {
 
 	/* Figure out total percentage */
 	if (empty($overallTotal)) {
-	    $poData[$locale]['percentDone'] = 0;
+	    $poData[$locale]['percentDone'] = $poData[$locale]['exactPercentDone'] = 0;
 	} else {
-	    $poData[$locale]['percentDone'] = $pluginTotal * 100 / $overallTotal;
+	    $poData[$locale]['percentDone'] =
+		floor($pluginTotal * 100 * $pow / $overallTotal) / $pow;
+	    $poData[$locale]['exactPercentDone'] = $pluginTotal * 100 / $overallTotal;
 	}
 
 	foreach (array('translated', 'untranslated', 'fuzzy', 'obsolete') as $key) {
-	    $poData[$locale]['summary'][$key] = 100 * $summary[$locale][$key] / $overallTotal;
+	    $poData[$locale]['summary'][$key] =
+		floor($summary[$locale][$key] * 100 * $pow / $overallTotal) / $pow;
 	}
 	$poData[$locale]['summary']['total'] = $overallTotal;
     }
@@ -277,7 +288,7 @@ function parsePoFiles($poFiles) {
     /* Sort locales by overall total */
     uasort($poData, 'sortByPercentDone');
 
-    return array($poData, $mostRecentPoDate);
+    return array($poData, $mostRecentPoDate, $totalTranslated);
 }
 
 /**
@@ -288,8 +299,8 @@ function parsePoFiles($poFiles) {
  * regardless of percentage. When two entries are otherwise equal and both have a 'name' entry,
  * then the sort is by 'name' ASC.
  *
- * @param array  first entry to sort
- * @param array  second entry to sort
+ * @param array $a first entry to sort
+ * @param array $b second entry to sort
  * @return int   -1, 0, +1, depending on the comparision
  */
 function sortByPercentDone($a, $b) {
@@ -299,13 +310,13 @@ function sortByPercentDone($a, $b) {
 	return -1;
     }
 
-    if ($a['percentDone'] == $b['percentDone']) {
+    if ($a['exactPercentDone'] == $b['exactPercentDone']) {
 	if (isset($a['name']) && isset($b['name'])) {
 	    return ($a['name'] < $b['name']) ? -1 : 1;
 	}
 	return 0;
     }
-    return ($a['percentDone'] < $b['percentDone']) ? 1 : -1;
+    return ($a['exactPercentDone'] < $b['exactPercentDone']) ? 1 : -1;
 }
 
 function percentColor($percent) {
@@ -344,5 +355,4 @@ function modifier($string) {
 	return $string . '_dark';
     }
 }
-
 ?>
